/* *****************************************************************************
 * File:        dogl.c
 * Project:     Mini Anzeige Modul
 * Author:      Nicolas Meyertns
 * Version:     1.0.xc8
 * Web:         www.PIC-Projekte.de
 * ****************************************************************************/

#include <xc.h>
#include "font.h"
#include "dogl.h"

/*
 * Initialisierung des EA DOGL Displays mit dem SPI Bus
 */
void initDOGL(uint8_t *pContrast)
{
    A0 = COMMAND;   // Die folgenden Bytes sind Befehle

    sendSPI(0x40);  // Display start line set --> 0
    sendSPI(0xA1);  // ADC set --> reverse
    sendSPI(0xC0);  // Common output mode select --> normal
    sendSPI(0xA6);  // Display --> normal
    sendSPI(0xA2);  // LCD Bias set --> 1/9 (Duty 1/65)
    sendSPI(0x2F);  // Power control set --> Booster, Regulator and Follower on
    sendSPI(0xF8);  // Booster ratio set --> Set internal Booster to 4x
    sendSPI(0x00);  // ...
    sendSPI(0x27);  // Contrast set
    sendSPI(0x81);  // ...
    sendSPI(*pContrast);
    sendSPI(0xAC);  // Static indicator set --> no indicator
    sendSPI(0x00);  // ...
    sendSPI(0xAF);  // Display on/off

    clearDOGL();
}

/*
 * Lscht den Inhalt des Displays und springt danach wieder
 * auf die Position 0,0 (oben links)
 */
void clearDOGL(void)
{
    unsigned int k,adr;

    /*Displayinhalt lschen*/
    for(adr=0; adr<8; adr++)
    {
        A0=COMMAND;
        sendSPI(0xB0 + adr);
        sendSPI(0x10);
        sendSPI(0x00);
        A0=DATA;
        for(k=0; k<128; k++)
            sendSPI(0x00);
        A0=COMMAND;
    }

    sendSPI(0xB0);  // Sprung zu: Zeile 0, Spalte 0
    sendSPI(0x10);
    sendSPI(0x00);
    A0=DATA;
}

/*
 * Diese Funktion gibt eine Zeichenkette auf dem Bildschirm aus.
 * Mit einer Angabe der Adresse, kann dies an jedem beliebigen Ort auf
 * dem Display stattfinden. Des Weiteren kann zwischen normaler und
 * invertierter Darstellung gewhlt werden.
 *
 * page:        Adresse - Zeile (0..7)
 * column:      Adresse - Spalte (0..127)
 * inverse:     Invertierte Darstellung wenn true sonst normal
 * *pChain:     Die Zeichnkette ansich, welche geschrieben werden soll
 */
void charChainDOGL(uint8_t page, uint8_t column, bool inverse, const uint8_t *pChain)
{
    uint8_t k;

    setAddrDOGL(page,column);

    if(inverse)
        sendSPI(~font[0][1]);
    
    while(*pChain)
    {
        for(k=1; k <= FONTWIDTH; k++)
        {
            if(inverse)
                sendSPI(~font[(*pChain)-32][k]);
            else
                sendSPI(font[(*pChain)-32][k]);
        }

        if(inverse)
            sendSPI(~font[0][1]);
        else
            sendSPI(font[0][1]);
        pChain++;
    }
}

/*
 * Auswahl einer Adresse. Diese Funktion wird in der Regel nicht vom
 * Anwender selbst aufgerufen, sondern nur durch die Funktionen dieser
 * C-Datei.
 *
 * page:    Adresse - Zeile (0..7)
 * column:  Adresse - Spalte (0..127)
 */
void setAddrDOGL(uint8_t page, uint8_t column)
{
    if( page<8 && column<128 )
    {
        A0=COMMAND;
        sendSPI(0xB0 + page);
        sendSPI(0x10 + ((column&0xF0)>>4) );
        sendSPI(0x00 +  (column&0x0F) );
        A0=DATA;
    }
}

/*
 * Erhhen oder verringern des Kontrastes des DOGL Displays.
 *
 * increase:    Wenn true -> Erhhung des Kontrastes,
 *              wenn false -> Verringerung
 */
void setContrastDOGL(bool increase, uint8_t *pContrast)
{
    A0=COMMAND;

    if(increase)
    {
        if( (*pContrast+1) <= 63)
        {
            *pContrast++;
            sendSPI(0x81);
            sendSPI(*pContrast);
        }
    }
    else
    {
        if( (*pContrast-1) >= 0)
        {
            *pContrast--;
            sendSPI(0x81);
            sendSPI(*pContrast);
        }
    }

    A0=DATA;
}

/*
 * Funktion ist hnlich zu charChain, nur das es hier um Zeichen und Symbole
 * geht und nicht um Buchstaben. Mit den Makros aus der Datei (font.h) lsst
 * sich das zu zeichnende Zeichen auswhlen.
 *
 * Alle anderen Parameter sind wie bei charChain zu behandeln!
 */
void drawSignDOGL(uint8_t page, uint8_t column, bool inverse, uint8_t signIdex)
{
    uint8_t k;

    setAddrDOGL(page,column);

    for(k=1; k <= sign[signIdex][0]; k++)
    {
        if(inverse)
            sendSPI(~sign[signIdex][k]);
        else
            sendSPI(sign[signIdex][k]);
    }
}